/******************************************************************************
* C Include: Image                                                            
*                                                                             
* File:                                                                       
*    image.h                                                                  
*                                                                             
* Description:                                                                
*    Include file for the functions used to process image data.               
*                                                                             
* History:                                                                    
*    Created    : 20-AUG-93 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : 01-MAR-94 by J. Doggett,                                    
*       GetImage Version 1.0.                                                 
*                                                                             
*    Updated    : 31-MAR-95 by J. Doggett,                                    
*       Added prototype for new function Get_Tile_Size.                       
*    Updated    : 07-APR-95 by J. Doggett,                                    
*       Added prototype for new function printBriefCopyRight                  
*    Updated    : 13-JUL-95 by J. Doggett,                                    
*       Modified prototype for Decompress_Image function to reflect change in 
*       argument list.                                                        
*    Redelivered: 18-AUG-95 by J. Doggett,                                    
*       GetImage Version 1.2.                                                 
*                                                                             
*    Updated    : 24-NOV-96 by J. Doggett,				      
*       Modifed prototype of Extract_Image to reflect changes in argument list.
*    Redelivered: xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1993-1996 Association of Universities for Research in         
* Astronomy, Inc.  All Rights Reserved.                                       
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#ifndef __IMAGE__
#define __IMAGE__

#include "extrctn.h"
#include "generic.h"
#include "header.h"

/***************************
* Image derived data types *
***************************/
typedef short        IMAGE_VALUE;
typedef IMAGE_VALUE* IMAGE_DATA ;

enum image_format { FITS, HHD };         /* Image format for output */
typedef enum image_format IMAGE_FORMAT;  /*    extractions          */

/******************************
* Image functions' prototypes *
******************************/
HEADER     Construct_Image_Header    (/* HEADER       Plate_Header        ,
                                         EXTRACTION   Extraction          ,
                                         IMAGE_DATA   Image                */);

IMAGE_DATA Decompress_Image          (/* EXTRACTION   Extraction          ,
                                         STRING       File_Name_Template  ,
                                         int          plateWidthPixels    ,
                                         int          plateHeightPixels    */);

/****************************
* Image include file page 2 *
****************************/

void       Extract_Image             (/* EXTRACTION   Extraction          ,
                                         HEADER       Plate_Header        ,
                                         STRING       Image_Data_Root_Dir ,
                                         STRING       imageFileRoot       ,
                                         IMAGE_FORMAT Format              ,
					 STRING       calibrationFile      */);

BOOL       FITS_File_Exists          (/* STRING       imageFileRoot        */);

void       Get_Image_Range           (/* IMAGE_DATA   Image               ,
                                         IMAGE_DATA   Data_Min            ,
                                         IMAGE_DATA   Data_Max             */);

void       Get_Tile_Size             (/* STRING       Tile_File_Name      ,
                                         int       * pWidth
                                         int       * pHeight               */);

BOOL       HHD_File_Exists           (/* STRING       imageFileRoot        */);

BOOL       HHH_File_Exists           (/* STRING       imageFileRoot        */);


BOOL       Image_File_Exists         (/* STRING       imageFileRoot       ,
                                         IMAGE_FORMAT Format               */);

STRING     Make_Plate_File_Name_Template
                                     (/* STRING       Image_Data_Directory,
                                         STRING       Plate_Name           */);

void       printBriefCopyRight       (/* HEADER       Plate_Header        ,
                                         FILE       * outputFile           */);

void       Set_Tile_File_Name_Extension
                                     (/* STRING       FileName            ,
                                         char         ExtensionCharacter1 ,
                                         char         ExtensionCharacter2  */);

#endif
