/******************************************************************************
* C Include: Lookup Table                                                     
*                                                                             
* File:                                                                       
*    looktab.h                                                                
*                                                                             
* Description:                                                                
*    Include file for the Lookup Table functions.                             
*                                                                             
* History:                                                                    
*    Created    : 12-SEP-96 by J. Doggett,                                    
*       for the Association of Universities for Research in Astronomy, Inc.   
*          at the Space Telescope Science Institute,                          
*       to facilitate the accessing of data from the CD-ROM set of the        
*          Digitized Sky Survey.                                              
*    Delivered  : xx-xxx-xx by J. Doggett,                                    
*       GetImage Version 2.0x.                                                
*                                                                             
* Copyright (c) 1996, Association of Universities for Research in Astronomy,  
* Inc.  All Rights Reserved.                                                  
* Produced under National Aeronautics and Space Administration grant          
* NAG W-2166.                                                                 
******************************************************************************/
#ifndef __LOOKTAB__
#define __LOOKTAB__

#include "lnkd_lst.h"

/**********************************
* Lookup Table derived data types *
**********************************/
struct Lookup_Table_Entry
{
   char * independent_value;
   char * dependent_value  ;
};
typedef struct Lookup_Table_Entry * LOOK_TAB_ENTRY;

typedef LL      LOOK_TAB     ;
typedef LL_NODE LOOK_TAB_NODE;

/*************************************
* Lookup Table functions' prototypes *
*************************************/
int             Add_LookupTbl_Entry_to_Tbl
                                    (/* LOOK_TAB         table            ,
					char           * independent_value,
					char           * dependent_value   */);
							    
int             Cmp_LookupTbl_EntryIndepndVal
                                    (/* LOOK_TAB_ENTRY   entry, 
					char           * input_value       */);
   
LOOK_TAB        Create_LookupTbl    (/*                                    */);
							    
void            Del_LookupTbl       (/* LOOK_TAB         table             */);
							    
void            Del_LookupTbl_Entry_from_Tbl
                                    (/* LOOK_TAB         table            ,
					char           * independent_value */);
							    
void            Del_LookupTbl_Entry (/* LOOK_TAB_ENTRY   entry             */);
   
char          * Find_LookupTbl_Entry(/* LOOK_TAB         table            ,
					char           * independent_value */);

/* *** Lookup Table Include File ***                                       2 */


#endif
