/****************************************************************************/
/*                      MODULE: client.c                                    */
/*                                                                          */
/* ROLE : Fonctions permettant de dialoguer avec le serveur de SIMBAD       */
/*                                                                          */
/* IMPORTATION :                                                            */
/*               libsk.a						    */
/*                                                                          */
/* EXPORTATIONS :                                                           */
/*                                                                          */
/* FONCTIONS LOCALES :                                                      */
/*                                                                          */
/* AUTEURS : Eric DIVETAIN ( Observatoire astronomique de Strasbourg )      */
/*                                                                          */
/* DATE DE CREATION : 15/07/93                                              */
/*                                                                          */
/* MODIFICATIONS : MW - fev 94 :                                            */
/*			envoi d'un code application et			    */
/*			test du numero de version			    */
/*                                                                          */
/****************************************************************************/
#define VERSION "V310"
/****************************************************************************/
/* INCLUDES SYSTEMES                                                        */
/****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

/****************************************************************************/
/* INCLUDES UTILISATEUR                                                     */
/****************************************************************************/
#include <simclic.h>

/****************************************************************************/
/* CONSTANTES                                                               */
/****************************************************************************/
#define FBUF 100
#define	HEAD_BUF 8		/* Taille de l'entete des chaines echangees */
#define OKSUITE "OKSUITE"	/* utilise dans ClientTreat() */
#define PASUITE "PASUITE"
#define AGAIN	"AGAIN"
#define UIF_CONTROLE	20	/* Code de controle interne a l'uif */

#define CODSIMBAD "1"
#define CODTRANFI "3"

/****************************************************************************/
/* TYPES                                                                    */
/****************************************************************************/

/****************************************************************************/
/* VARIABLES                                                                */
/****************************************************************************/
static char *sendstring; /* Pointeur sur la chaine a envoyer ClientSend() */
static int stringsize = 0;/* Longueur de la chaine a envoyer ClientSend() */
static char *buffer;	/* Pointeur sur le buffer recu ClientGet() */
static int  bufsize = 0;/* Longueur de la chaine recue ClientGet() */
static char dirpath[10];
static char statserveur[20];
static char statservice[20];
static char statname[20];
static char statpasswd[20];

/****************************************************************************/
/* FONCTIONS IMPORTEES                                                      */
/****************************************************************************/
/* CG. 10.09.07 malloc() is predefined */
/* extern char *malloc(); */
extern int sk_connect();	/* /usr/local/lib/libsk.a */
extern int sk_puts();
extern int sk_get();
extern int sk_gets();
extern int sk_close();

/****************************************************************************/
/* FONCTIONS EXPORTEES                                                      */
/****************************************************************************/
int ClientOpen();
int ClientTreat();
void ClientClose();

/****************************************************************************/
/* FONCTIONS LOCALES                                                        */
/****************************************************************************/
static int ClientSend();
static int ClientGet();

/****************************************************************************/
/* FONCTION :	                                                            */
/*	static int ClientConnect( display_fct, host, service, name, passwd )*/
/* PRAMETRES :                                                              */
/*      -ENTREES :                                                          */
/*		char *host: nom du serveur                                  */
/*              char *service: nom du service                               */
/*              char *name: nom different du userid                         */
/*              char *passwd: password                                      */
/*      -SORTIES :                                                          */
/*		void (*display_fct)(): Fonction de traitement des messages  */
/*		retourne: ERROR ou ear (numero de socket)		    */
/* ROLE :                                                                   */
/*	Cette fonction permet de se connecter au serveur de SIMBAD, et      */
/* retourne le numero de socket                                             */
/*--------------------------------------------------------------------------*/
/* Realisation :                                                            */
/****************************************************************************/

	static int ClientConnect( display_fct, host, service, name, passwd )

	void
		(*display_fct)();
	char
		*host,
		*service,
		*name,
		*passwd;
{
	int
		ear;
	SimDatStruct
		teststruct;

	teststruct.code = 0;
	teststruct.msg = 0;
	teststruct.length = 0;

	ear = sk_connect( host, service, name, passwd );
	if( ear < 0 )
	{
		teststruct.code = UIF_CONNEXIONERROR;
		teststruct.msg = ERR_SOCKET;
		if( display_fct != NULL )
		{
			display_fct( &teststruct );
		}
		return( ERROR );
	}
	return( ear );
}

/****************************************************************************/
/* FONCTION :                                                               */
/*	int ClientOpen( display_fct, host, service, name, passwd, appli )   */
/* PRAMETRES :                                                              */
/*      -ENTREES :                                                          */
/*		char *host: nom du serveur 				    */
/*		char *service: nom du service 				    */
/*		char *name: nom different du userid 			    */
/*		char *passwd: password					    */	
/*		char *appli: nom de l'application			    */
/*      -SORTIES :                                                          */
/*		void (*display_fct)(): Fonction de traitement des messages  */
/*		retourne: ERROR ou BAD_VERSION ou ear (numero de socket)    */
/* ROLE :                                                                   */
/*	Cette fonction permet de se connecter au serveur de SIMBAD, et      */
/* transmet le motd a l'aide de la fonction 'display_fct()' qui a pour      */
/* argument l'adresse d'une structure du type 'SimDatStruct'                */
/* ( voir client.h ). Cette fonction envoie le code associe au service      */
/* SIMBAD		                                                    */	
/*--------------------------------------------------------------------------*/
/* Realisation :                                                            */
/****************************************************************************/

	int ClientOpen( display_fct, host, service, name, passwd, appli )

	void
		(*display_fct)();
	char
		*host,
		*service,
		*name,
		*passwd,
		*appli;
{
	SimDatStruct
		getstruct,
		welcome_msg;
	int
		ear;
	void
		ClientInterrupt();
	char
		constr[10],
		ctrl_str[256],
		version_ack[8];
	int
		ctrl_str_lg,
		version_ack_lg ;

	strcpy( statname, name);
	strcpy( statpasswd, passwd);
	strcpy( statserveur, host);
	strcpy( statservice, service);
	welcome_msg.code = 0;
	welcome_msg.msg = 0;
	welcome_msg.length = 0;

	ear = sk_connect( host, service, name, passwd );
	if( ear < 0 )
	{
		welcome_msg.code = UIF_CONNEXIONERROR;
		welcome_msg.msg = ERR_SOCKET;
		welcome_msg.length = L_ERR_SOCKET;
		if( display_fct != NULL )
			display_fct( &welcome_msg );
		return( ERROR );
	}
	/* Envoi "version;appli" et test du numero de version */
	version_ack_lg = sizeof(version_ack) ;
	strcpy(ctrl_str,VERSION) ;
	strcat(ctrl_str,";") ;
	strcat(ctrl_str,appli) ;
	ctrl_str_lg = strlen(ctrl_str)+1 ;  /* +1 for the EOS char !!! */
	if (sk_write(ear, ctrl_str, ctrl_str_lg) != ctrl_str_lg ||
		 sk_read(ear,version_ack,version_ack_lg) <= 0 ||
		 strcmp(version_ack,"OK") != 0)
	{
		welcome_msg.code = UIF_CONNEXIONERROR;
		welcome_msg.msg = ERR_VERSION ;
		welcome_msg.length = L_ERR_VERSION ;
		if( display_fct != NULL )
			display_fct( &welcome_msg );
		return(BAD_VERSION) ;
	}

	if( ClientSend( ear, CODSIMBAD ) < 0 )
	{
		welcome_msg.code = UIF_CONNEXIONERROR;
		welcome_msg.msg = ERR_WRITE;
		welcome_msg.length = L_ERR_WRITE;
		if( display_fct != NULL )
			display_fct( &welcome_msg );
		return( ERROR );
	}

	if( ClientGet( ear, &welcome_msg) < 0 )
	{
		welcome_msg.code = UIF_CONNEXIONERROR;
		welcome_msg.msg = ERR_READ;
		welcome_msg.length = L_ERR_READ;
		if( display_fct != NULL )
			display_fct( &welcome_msg );
		return( ERROR );
	}
	strcpy( dirpath, welcome_msg.msg);

	if( ClientGet( ear, &welcome_msg) < 0 )
	{
		welcome_msg.code = UIF_CONNEXIONERROR;
		welcome_msg.msg = ERR_READ;
		welcome_msg.length = L_ERR_READ;
		if( display_fct != NULL )
			display_fct( &welcome_msg );
		return( ERROR );
	}

	if( display_fct != NULL )
		display_fct( &welcome_msg );

	return( ear );
}

/****************************************************************************/
/* FONCTION :                                                               */
/*	int ClientSend( ear, ask_string )				    */
/* PRAMETRES :                                                              */
/*      -ENTREES :                                                          */
/*		int ear: numero de socket				    */
/*		char *ask_string : requete				    */
/*      -SORTIES :                                                          */
/*		retourne: OK, ERROR, CONNEXIONERROR			    */
/* ROLE :                                                                   */
/*--------------------------------------------------------------------------*/
/* Realisation :                                                            */
/*	Cette fonction permet d'envoyer une chaine de caractere associee    */
/* a la structure SimDatStruct (client.h)				    */
/****************************************************************************/

	static int ClientSend( ear, askstring )

	int
		ear;
	char
		*askstring;
{
	int
		asksize;

	asksize = strlen( askstring );
	if( asksize + HEAD_BUF + 1 >= stringsize )
	{
		if( stringsize != 0 ) 
		{
			free(sendstring);
		}
		stringsize = asksize + HEAD_BUF + 1;
		stringsize = (stringsize+511)&~511;

		sendstring = malloc( stringsize );
		if( sendstring == NULL )
		{
			perror("malloc() in ClientSend()");
			return( ERROR );
		}
	}
	sprintf(sendstring,"%c%c%6d%s", '*','*', asksize, askstring);

	if( sk_put( ear, sendstring ) == ERROR )
	{
		return( CONNEXIONERROR );
	}
	return( OK );
}

/****************************************************************************/
/* FONCTION : ClientGet                                                     */
/* PRAMETRES :                                                              */
/*      -ENTREES : 							    */	
/*		int ear : numero de socket                                  */
/*		SimDatStruct getstruct : structure initialisee		    */
/*      -SORTIES :                                                          */
/*		retourne : OK, CONNEXIONERROR, NOEND                        */
/* ROLE :								    */
/* 	Cette fonction recupere la chaine de caractere se trouvant sur le   */
/* socket ear et initialise la structure getstruct                          */
/*--------------------------------------------------------------------------*/
/* Realisation :                                                            */
/****************************************************************************/

	static int ClientGet( ear, getstruct )

	SimDatStruct
		*getstruct;
	int
		ear;
{

	char
		head_string[HEAD_BUF + 1],
		type[3],
		flag;
	int
		lenght = 0;

	getstruct->msg = 0;

	if( sk_get( ear, head_string, HEAD_BUF ) == ERROR )
	{
		return( CONNEXIONERROR );
	}
	head_string[HEAD_BUF] = '\0';

	type[0] = head_string[0];
	type[1] = head_string[1];
	type[2] = '\0';
	getstruct->code =  atoi(type);
	flag = head_string[2];

	lenght = atoi( &(head_string[3]) );
	getstruct->length = lenght;

	if (lenght >= bufsize)
	{
		if(buffer)
		{
			free(buffer);
		}

		bufsize = lenght + 8 ;
	    	bufsize = (bufsize+511)&~511;

	    	buffer = malloc(bufsize);
	    	if( buffer == NULL )
	    	{
			perror("malloc() in ClientGet()");
			return( ERROR );
	    	}
	}
	getstruct->msg = buffer;

	if( sk_get( ear, getstruct->msg, lenght) == ERROR )
	{
		return( CONNEXIONERROR );
	}
	getstruct->msg[lenght] = '\0';

	if( flag == 'e' )
	{
		return( OK );
	}
	else if( flag == 's' )
	{
		return( NOEND );
	}
}

/****************************************************************************/
/* FONCTION :                                                               */
/*	int ClientTreat( ear, askstring, display_fct, code )		    */
/* PRAMETRES :                                                              */
/*      -ENTREES :                                                          */
/*		int ear : numero de socket				    */
/*		char *askstring : pointeur sur la requete		    */
/*		void (*display_fct)() : fonction d'affichage 		    */
/*		int code : argument de la fonction display_fct()	    */
/*      -SORTIES :                                                          */
/*		retourne : OK, CONNEXIONERROR, NOEND			    */
/* ROLE :                                                                   */
/*	Cette fonction envoie la requete 'askstring' au serveur et renvoie  */
/* le resultat a travers la fonction 'display_fct()' qui a pour argument    */
/* l'adresse d'une structure du type 'SimDatStruct' (Server.h)   	    */
/*--------------------------------------------------------------------------*/
/* Realisation :                                                            */
/****************************************************************************/

	int ClientTreat( ear, askstring, display_fct, code )

	void
		(*display_fct)();
	char
		*askstring;
	int
		ear,
		code;
{
	int
		suite;

	SimDatStruct
		getstruct;

	if( strcmp( askstring, "" ) == 0 )
	{
		askstring = OKSUITE;
	}

	if( ClientSend( ear, askstring ) < 0 )
	{
		return( CONNEXIONERROR );
	}
	
	suite = ClientGet( ear, &getstruct );

	if( strcmp( getstruct.msg, AGAIN ) == (int)NULL )
	{
		ClientTreat( ear, askstring, display_fct, code );
	}
	else
	{
		display_fct( &getstruct, code );
		return( suite );
	}
}

/****************************************************************************/
/* FONCTION :                                                               */
/*		void ClientClose( ear )					    */
/* PRAMETRES :                                                              */
/*      -ENTREES :                                                          */
/*		int ear: numero de socket				    */
/*      -SORTIES :                                                          */
/* ROLE :                                                                   */
/*	Cette fonction libere le socket ear				    */
/*--------------------------------------------------------------------------*/
/* Realisation :                                                            */
/****************************************************************************/

	void ClientClose( ear )

	int
		ear;
{
	sk_close( ear );
}

/***********************************************************************

 NAME ReadFormat( )

 DESCRIPTION

	Cette fonction permet de lire un fichier dans lequel se trouve
un format et de le transmettre au serveur

 RETURN VALUES
	OK/ERROR

************************************************************************/
	int ClientTransFile( display_fct, fullname, name )

	char
		*fullname,
		*name;
	void
		(*display_fct)();
{
	SimDatStruct
		errormsg;
	int
		ear,
		file;
	char
		newname[50];

	errormsg.code = 0;
	errormsg.msg = 0;
	errormsg.length = 0;

	ear = ClientConnect( display_fct, statserveur, statservice, statname, statpasswd );
	if( ear < 0 )
        {
                return( ERROR );
        }

        if( ClientSend( ear, CODTRANFI ) < 0 )
	{
		errormsg.code = UIF_CONNEXIONERROR;
		errormsg.msg = ERR_WRITE;
		errormsg.length = L_ERR_WRITE;
		display_fct( &errormsg );
		return( ERROR );
	}

	sprintf(newname, "./%s/%s", dirpath, name);
        if( ClientSend( ear, newname ) < 0 )
	{
		errormsg.code = UIF_CONNEXIONERROR;
		errormsg.msg = ERR_WRITE;
		errormsg.length = L_ERR_WRITE;
		display_fct( &errormsg );
		return( ERROR );
	}

	file = open( fullname, 0 );
	if( file < 0 )
	{
		errormsg.code = UIF_OTHERERROR;
		errormsg.msg = ERR_OPEN_FILE;
		errormsg.length = L_ERR_OPEN_FILE;
		display_fct( &errormsg );
		return( ERROR );
	}

	sk_fromserver( ear, 1, file );
	close( file );
        return( OK );
}

